@@ -103,7 +103,7 @@
 //! if run_cmd! {
 //!     cat ${file} | grep ${keyword};
 //!     echo "bad cmd" >&2;
-//!     ls /nofile || true;
+//!     ignore ls /nofile;
 //!     date;
 //!     ls oops;
 //!     cat oops;
@@ -202,7 +202,7 @@
 //!
 //! ### Builtin commands
 //! #### cd
-//! cd: set process current directory, which is always enabled
+//! cd: set process current directory, which is always enabled.
 //! ```no_run
 //! # use cmd_lib::run_cmd;
 //! run_cmd! (
@@ -218,15 +218,15 @@
 //! Use `std::env::set_current_dir` if you want to change the current
 //! working directory for the whole program.
 //!
-//! #### true
+//! #### ignore
 //!
-//! Just return true without launching any processes.
+//! Ignore errors for command executiona, which is always enabled.
 //!
 //! #### echo
 //!
 //! ```
 //! # use cmd_lib::{run_cmd, use_builtin_cmd};
-//! use_builtin_cmd!(true, echo); // find more builtin commands in src/builtins.rs
+//! use_builtin_cmd!(echo); // find more builtin commands in src/builtins.rs
 //! run_cmd!(echo "This is from builtin command!")?;
 //! # Ok::<(), std::io::Error>(())
 //! ```
@@ -255,12 +255,12 @@
 //! `spawn!()` macro executes the whole command as a child process, returning a handle to it. By
 //! default, stdin, stdout and stderr are inherited from the parent.
 //!
-//! To get result, you can call `wait_cmd_result()` or `wait_fun_result()` to get CmdResult/FunResult.
+//! With `spawn_with_output!()` you can get output result by calling `wait_fun_result()`.
 //!
 //! ```no_run
-//! # use cmd_lib::spawn;
+//! # use cmd_lib::*;
 //! spawn!(ping -c 10 192.168.0.1)?.wait_cmd_result()?;
-//! let output = spawn!(/bin/cat file.txt | sed s/a/b/)?.wait_fun_result()?;
+//! let output = spawn_with_output!(/bin/cat file.txt | sed s/a/b/)?.wait_fun_result()?;
 //! # Ok::<(), std::io::Error>(())
 //! ```
 //!
@@ -322,7 +322,7 @@
 
 pub use cmd_lib_macros::{
     cmd_debug, cmd_die, cmd_echo, cmd_error, cmd_info, cmd_trace, cmd_warn, export_cmd, run_cmd,
-    run_fun, spawn, use_builtin_cmd, use_custom_cmd,
+    run_fun, spawn, spawn_with_output, use_builtin_cmd, use_custom_cmd,
 };
 /// Return type for run_fun!() macro
 pub type FunResult = std::io::Result<String>;
@@ -330,7 +330,7 @@ pub type FunResult = std::io::Result<String>;
 pub type CmdResult = std::io::Result<()>;
 pub use builtins::{
     builtin_cat, builtin_debug, builtin_die, builtin_echo, builtin_error, builtin_info,
-    builtin_trace, builtin_true, builtin_warn,
+    builtin_trace, builtin_warn,
 };
 pub use child::CmdChildren;
 #[doc(hidden)]
