@@ -1,6 +1,6 @@
 use crate::child::{CmdChild, CmdChildren};
 use crate::io::{CmdIn, CmdOut};
-use crate::{builtin_true, CmdResult, FunResult};
+use crate::{CmdResult, FunResult};
 use faccess::{AccessMode, PathExt};
 use lazy_static::lazy_static;
 use log::{debug, error};
@@ -60,8 +60,7 @@ type FnFun = fn(&mut CmdEnv) -> CmdResult;
 lazy_static! {
     static ref CMD_MAP: Mutex<HashMap<OsString, FnFun>> = {
         // needs explicit type, or it won't compile
-        let mut m: HashMap<OsString, FnFun> = HashMap::new();
-        m.insert("".into(), builtin_true);
+        let m: HashMap<OsString, FnFun> = HashMap::new();
         Mutex::new(m)
     };
 }
@@ -88,29 +87,21 @@ pub fn set_pipefail(enable: bool) {
 #[doc(hidden)]
 #[derive(Default)]
 pub struct GroupCmds {
-    group_cmds: Vec<(Cmds, Option<Cmds>)>, // (cmd, orCmd) pairs
+    group_cmds: Vec<Cmds>,
     current_dir: PathBuf,
 }
 
 impl GroupCmds {
-    pub fn add(mut self, cmds: Cmds, or_cmds: Option<Cmds>) -> Self {
-        self.group_cmds.push((cmds, or_cmds));
+    pub fn append(mut self, cmds: Cmds) -> Self {
+        self.group_cmds.push(cmds);
         self
     }
 
     pub fn run_cmd(&mut self) -> CmdResult {
         for cmds in self.group_cmds.iter_mut() {
-            if let Err(err) = cmds.0.run_cmd(&mut self.current_dir) {
-                if let Some(or_cmds) = &mut cmds.1 {
-                    let ret = or_cmds.run_cmd(&mut self.current_dir);
-                    if let Err(err) = ret {
-                        error!("Running {} failed, Error: {}", or_cmds.get_full_cmds(), err);
-                        return Err(err);
-                    }
-                } else {
-                    error!("Running {} failed, Error: {}", cmds.0.get_full_cmds(), err);
-                    return Err(err);
-                }
+            if let Err(err) = cmds.run_cmd(&mut self.current_dir) {
+                error!("Running {} failed, Error: {}", cmds.get_full_cmds(), err);
+                return Err(err);
             }
         }
         Ok(())
@@ -120,28 +111,20 @@ impl GroupCmds {
         let mut last_cmd = self.group_cmds.pop().unwrap();
         self.run_cmd()?;
         // run last function command
-        let ret = last_cmd.0.run_fun(&mut self.current_dir);
+        let ret = last_cmd.run_fun(&mut self.current_dir);
         if let Err(e) = ret {
-            if let Some(or_cmds) = &mut last_cmd.1 {
-                let or_ret = or_cmds.run_fun(&mut self.current_dir);
-                if let Err(ref err) = or_ret {
-                    error!("Running {} failed, Error: {}", or_cmds.get_full_cmds(), err);
-                }
-                or_ret
-            } else {
-                let full_cmds = last_cmd.0.get_full_cmds();
-                error!("Running {} failed, Error: {}", full_cmds, e);
-                Err(e)
-            }
+            let full_cmds = last_cmd.get_full_cmds();
+            error!("Running {} failed, Error: {}", full_cmds, e);
+            Err(e)
         } else {
             ret
         }
     }
 
-    pub fn spawn(mut self) -> Result<CmdChildren> {
+    pub fn spawn(mut self, with_output: bool) -> Result<CmdChildren> {
         assert_eq!(self.group_cmds.len(), 1);
-        let mut cmds = self.group_cmds.pop().unwrap().0;
-        let ret = cmds.spawn(&mut self.current_dir);
+        let mut cmds = self.group_cmds.pop().unwrap();
+        let ret = cmds.spawn(&mut self.current_dir, with_output);
         if let Err(ref err) = ret {
             error!("Spawning {} failed, Error: {}", cmds.get_full_cmds(), err);
         }
@@ -170,7 +153,7 @@ impl Cmds {
         &self.full_cmds
     }
 
-    fn spawn(&mut self, current_dir: &mut PathBuf) -> Result<CmdChildren> {
+    fn spawn(&mut self, current_dir: &mut PathBuf, with_output: bool) -> Result<CmdChildren> {
         if std::env::var("CMD_LIB_DEBUG") == Ok("1".into()) {
             debug!("Running {} ...", self.get_full_cmds());
         }
@@ -189,7 +172,7 @@ impl Cmds {
             } else {
                 cmd.setup_redirects(&mut last_pipe_in, None)?;
             }
-            let child = cmd.spawn(current_dir)?;
+            let child = cmd.spawn(current_dir, with_output)?;
             children.push(child);
         }
 
@@ -197,11 +180,11 @@ impl Cmds {
     }
 
     fn run_cmd(&mut self, current_dir: &mut PathBuf) -> CmdResult {
-        self.spawn(current_dir)?.wait_cmd_result_nolog()
+        self.spawn(current_dir, false)?.wait_cmd_result_nolog()
     }
 
     fn run_fun(&mut self, current_dir: &mut PathBuf) -> FunResult {
-        self.spawn(current_dir)?.wait_fun_result_nolog()
+        self.spawn(current_dir, true)?.wait_fun_result_nolog()
     }
 }
 
@@ -244,6 +227,7 @@ pub struct Cmd {
     args: Vec<OsString>,
     vars: HashMap<String, String>,
     redirects: Vec<Redirect>,
+    ignore_error: bool,
 
     // for running
     std_cmd: Option<Command>,
@@ -261,6 +245,7 @@ impl Default for Cmd {
             args: vec![],
             vars: HashMap::new(),
             redirects: vec![],
+            ignore_error: false,
             std_cmd: None,
             stdin_redirect: None,
             stdout_redirect: None,
@@ -275,6 +260,10 @@ impl Cmd {
     pub fn add_arg(mut self, arg: OsString) -> Self {
         let arg_str = arg.to_string_lossy().to_string();
         if self.args.is_empty() {
+            if arg_str == "ignore" {
+                self.ignore_error = true;
+                return self;
+            }
             let v: Vec<&str> = arg_str.split('=').collect();
             if v.len() == 2 && v[0].chars().all(|c| c.is_ascii_alphanumeric() || c == '_') {
                 self.vars.insert(v[0].into(), v[1].into());
@@ -337,7 +326,7 @@ impl Cmd {
         self
     }
 
-    fn spawn(mut self, current_dir: &mut PathBuf) -> Result<CmdChild> {
+    fn spawn(mut self, current_dir: &mut PathBuf, with_output: bool) -> Result<CmdChild> {
         let arg0 = self.arg0();
         let full_cmd = self.debug_str();
         if arg0 == "cd" {
@@ -386,10 +375,10 @@ impl Cmd {
                     stdout: self.stdout_logging,
                     stderr: self.stderr_logging,
                     cmd: full_cmd,
+                    ignore_error: self.ignore_error,
                 })
             } else {
                 internal_cmd(&mut env)?;
-                drop(env);
                 Ok(CmdChild::SyncFn {
                     cmd: full_cmd,
                     stdout: self.stdout_logging,
@@ -412,7 +401,7 @@ impl Cmd {
             // update stdout
             if let Some(redirect_out) = self.stdout_redirect.take() {
                 cmd.stdout(redirect_out);
-            } else {
+            } else if with_output {
                 cmd.stdout(Stdio::piped());
             }
 
@@ -426,6 +415,7 @@ impl Cmd {
             Ok(CmdChild::Proc {
                 cmd: full_cmd,
                 stderr: self.stderr_logging,
+                ignore_error: self.ignore_error,
                 child,
             })
         }
